@@ -1,7 +1,7 @@
 from __future__ import unicode_literals
 
 from django.core import checks
-from django.db import IntegrityError, router
+from django.db import IntegrityError, connections, router
 from django.db.models import CASCADE
 from django.db.models.fields.related import ForeignKey, ManyToManyField
 from django.utils.functional import cached_property
@@ -69,7 +69,10 @@ def create_deferring_foreign_related_manager(related, original_manager_cls):
         def _apply_rel_filters(self, queryset):
             # Implemented as empty for compatibility sake
             # But there is probably a better implementation of this function
-            return queryset._next_is_sticky()
+            #
+            # NOTE: _apply_rel_filters() must return a copy of the queryset
+            # to work correctly with prefetch
+            return queryset._next_is_sticky().all()
 
         def get_prefetch_queryset(self, instances, queryset=None):
             if queryset is None:
@@ -271,14 +274,19 @@ class ParentalKey(ForeignKey):
 
 
 def create_deferring_forward_many_to_many_manager(rel, original_manager_cls):
-    relation_name = rel.field.name
+    rel_field = rel.field
+    relation_name = rel_field.name
+    query_field_name = rel_field.related_query_name()
+    source_field_name = rel_field.m2m_field_name()
     rel_model = rel.model
     superclass = rel_model._default_manager.__class__
+    rel_through = rel.through
 
     class DeferringManyRelatedManager(superclass):
         def __init__(self, instance=None):
             super(DeferringManyRelatedManager, self).__init__()
             self.model = rel_model
+            self.through = rel_through
             self.instance = instance
 
         def get_original_manager(self):
@@ -317,6 +325,46 @@ def create_deferring_forward_many_to_many_manager(rel, original_manager_cls):
 
             return FakeQuerySet(rel_model, results)
 
+        def get_prefetch_queryset(self, instances, queryset=None):
+            # Derived from Django's ManyRelatedManager.get_prefetch_queryset.
+            if queryset is None:
+                queryset = super().get_queryset()
+
+            queryset._add_hints(instance=instances[0])
+            queryset = queryset.using(queryset._db or self._db)
+
+            query = {'%s__in' % query_field_name: instances}
+            queryset = queryset._next_is_sticky().filter(**query)
+
+            fk = self.through._meta.get_field(source_field_name)
+            join_table = fk.model._meta.db_table
+
+            connection = connections[queryset.db]
+            qn = connection.ops.quote_name
+
+            queryset = queryset.extra(select={
+                '_prefetch_related_val_%s' % f.attname:
+                '%s.%s' % (qn(join_table), qn(f.column)) for f in fk.local_related_fields})
+
+            return (
+                queryset,
+                lambda result: tuple(
+                    getattr(result, '_prefetch_related_val_%s' % f.attname)
+                    for f in fk.local_related_fields
+                ),
+                lambda inst: tuple(
+                    f.get_db_prep_value(getattr(inst, f.attname), connection)
+                    for f in fk.foreign_related_fields
+                ),
+                False,
+                relation_name,
+                False,
+            )
+
+        def _apply_rel_filters(self, queryset):
+            # Required for get_prefetch_queryset.
+            return queryset._next_is_sticky()
+
         def get_object_list(self):
             """
             return the mutable list that forms the current in-memory state of
