@@ -2,6 +2,7 @@ from __future__ import unicode_literals
 
 import unittest
 
+from django import VERSION as DJANGO_VERSION
 from django.core.exceptions import ValidationError
 from django.test import TestCase
 from tests.models import Band, BandMember, Album, Restaurant, Article, Author, Document, Gallery, Song
@@ -169,6 +170,31 @@ class ClusterFormTest(TestCase):
         form = BandForm()
         self.assertTrue(isinstance(form.formsets.get("albums").forms[0], AlbumForm))
 
+    def test_alternative_formset_name(self):
+        """Support specifying a formset_name that differs from the relation"""
+        class BandForm(ClusterForm):
+            class Meta:
+                model = Band
+                formsets = {
+                    'albums': {'fields': ['name'], 'formset_name': 'records'}
+                }
+                fields = ['name']
+
+        form = BandForm({
+            'name': "The Beatles",
+
+            'records-TOTAL_FORMS': 1,
+            'records-INITIAL_FORMS': 0,
+            'records-MAX_NUM_FORMS': 1000,
+
+            'records-0-name': 'Please Please Me',
+            'records-0-id': '',
+        })
+
+        self.assertTrue(form.is_valid())
+        result = form.save(commit=False)
+        self.assertEqual(result.albums.first().name, 'Please Please Me')
+
     def test_formfield_callback(self):
 
         def formfield_for_dbfield(db_field, **kwargs):
@@ -308,8 +334,12 @@ class ClusterFormTest(TestCase):
             'members-2-id': '',
         }, instance=beatles)
 
-        with self.assertRaises(ValidationError):
-            form.is_valid()
+        if DJANGO_VERSION >= (3, 2):
+            # in Django >=3.2, a missing ManagementForm gives a validation error rather than an exception
+            self.assertFalse(form.is_valid())
+        else:
+            with self.assertRaises(ValidationError):
+                form.is_valid()
 
     def test_saved_items_with_non_db_relation(self):
         class BandForm(ClusterForm):
